# vim:ts=4:sw=4:ft=python:fileencoding=utf-8
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

"""Checks an RSS feed and posts new entries to twitter."""

# standard libraires imports
from configparser import SafeConfigParser, NoOptionError, NoSectionError
from argparse import ArgumentParser
import codecs
import logging
import os
import sys

# 3rd party libraries imports
import feedparser
import tweepy

class TweetPost(object):
    '''TweetPost class'''

    def __init__(self, config, tweet):
        '''Constructore of the TweetPost class'''
        self.config = config
        self.tweet = tweet
        self.main()

    def main(self):
        '''Main of the TweetPost class'''
        consumer_key = self.config.get('twitter', 'consumer_key')
        consumer_secret = self.config.get('twitter', 'consumer_secret')
        access_token = self.config.get('twitter', 'access_token')
        access_token_secret = self.config.get('twitter', 'access_token_secret')
        auth = tweepy.OAuthHandler(consumer_key, consumer_secret)
        auth.set_access_token(access_token, access_token_secret)
        api = tweepy.API(auth)
        try:
            api.update_status(self.tweet)
        except(tweepy.error.TweepError) as e:
            logging.warning("Error occurred while updating status: %s", e)
        else:
            return True




# -*- coding: utf-8 -*-
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Filter an entry of the RSS feeds
'''Filter an entry of the RSS feeds'''

# standard library imports
from configparser import SafeConfigParser, NoOptionError, NoSectionError
import os
import os.path
import sys

# 3rd party library imports
import feedparser

class FilterEntry(object):
    '''FilterEntry class'''
    def __init__(self, elements, entry, options):
        '''Constructor of the FilterEntry class'''
        self.matching = {}
        self.entry = entry
        self.elements = elements
        self.options = options
        self.main()

    def main(self):
        '''Main of the FilterEntry class'''
        for i in self.elements:
            if i not in self.entry:
                sys.exit('The element {} is not available in the RSS feed. The available ones are: {}'.format(i, [j for j in self.entry]))
            if not self.options['patterns']:
                self.matching[i] = self.entry[i]
            else:
                # pattern matching on the title of the RSS feed
                for patternlist in self.options['patterns']:
                    for pattern in self.options['patterns'][patternlist]:
                        if not self.options['patternscasesensitive']['{}_case_sensitive'.format(patternlist)]:
                            # not case sensitive, so we compare the lower case
                            finalpattern = self.options['patterns'][pattern].lower() 
                            finaltitle = self.entry[pattern.split('_')[0]].lower()
                            if finalpattern in finaltitle:
                                self.matching[i] = self.entry[i]
                        else:
                            # case sensitive, so we use the user-defined pattern
                            if pattern in self.entry['title']:
                                self.matching[i] = self.entry[i]

    @property
    def finalentry(self):
        '''Return the processed entry'''
        return self.matching



#!/usr/bin/env python3
# vim:ts=4:sw=4:ft=python:fileencoding=utf-8
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

"""Checks an RSS feed and posts new entries to twitter."""

# standard libraires imports
import codecs
import importlib
import logging
import os
import sys

# 3rd party libraries imports
import feedparser
from persistentlist import PersistentList
import tweepy

# app libraries imports
from feed2tweet.addtags import AddTags
from feed2tweet.cliparse import CliParse
from feed2tweet.confparse import ConfParse
from feed2tweet.filterentry import FilterEntry
from feed2tweet.removeduplicates import RemoveDuplicates
from feed2tweet.tweetpost import TweetPost

class Main(object):
    '''Main class of Feed2tweet'''

    def __init__(self):
        self.main()

    def main(self):
        """The main function."""
        clip = CliParse()
        clioptions = clip.options
        logging.basicConfig(level=clioptions.log_level.upper(), format='%(message)s')
        # iterating over the different configuration files
        cfgp = ConfParse(clioptions)
        confs = cfgp.confvalues
        for conf in confs:
            options = conf[0]
            config = conf[1]
            tweetformat = conf[2]
            feeds = conf[3]
            plugins = conf[4]
            # open the persistent list
            cache = PersistentList(options['cachefile'][0:-3], 100)
            if options['hashtaglist']:
                severalwordshashtags = codecs.open(options['hashtaglist'],
                                                   encoding='utf-8').readlines()
                severalwordshashtags = [i.rstrip('\n') for i in severalwordshashtags]
            # fixing rss2twitter most old bug
            # reverse feed entries because most recent one should be sent as the last one in Twitter
            for feed in feeds:
                entries = feed['entries'][0:clioptions.limit]
                entries.reverse()
                # --rss-sections option: print rss sections and exit
                if clioptions.rsssections:
                    if entries:
                        print('The following sections are available in this RSS feed: {}'.format([j for j in entries[0]]))
                        sys.exit(0)
                    else:
                        sys.exit('Could not parse the section of the rss feed')
                totweet = []
                # cache the ids of last rss feeds
                if not clioptions.all:
                    for i in entries:
                        if i['id'] not in cache:
                            totweet.append(i)
                else:
                    totweet = entries

                for entry in totweet:
                    logging.debug('found feed entry %s, %s', entry['id'], entry['title'])


                    rss = {
                        'id': entry['id'],
                    }

                    severalwordsinhashtag = False
                    # lets see if the rss feed has hashtag
                    if 'tags' in entry:
                        hastags = True
                    else:
                        hastags = False

                    if hastags:
                        rss['hashtags'] = []
                        for i, _ in enumerate(entry['tags']):
                            if 'hashtaglist' in options:
                                prehashtags = entry['tags'][i]['term']
                                tmphashtags = entry['tags'][i]['term']
                                for element in severalwordshashtags:
                                    if element in prehashtags:
                                        severalwordsinhashtag = True
                                        tmphashtags = prehashtags.replace(element,
                                                                          ''.join(element.split()))
                            # replace characters stopping a word from being a hashtag
                            if severalwordsinhashtag:
                                # remove ' from hashtag
                                tmphashtags = tmphashtags.replace("'", "")
                                # remove - from hashtag
                                tmphashtags = tmphashtags.replace("-", "")
                                # remove . from hashtag
                                tmphashtags = tmphashtags.replace(".", "")
                                # remove space from hashtag
                                finalhashtags = tmphashtags.replace(" ", "")
                                rss['hashtags'].append('#{}'.format(finalhashtags))
                            else:
                                nospace = ''.join(entry['tags'][i]['term'])
                                # remove space from hashtag
                                nospace = nospace.replace(" ", "")
                                rss['hashtags'].append('#{}'.format(nospace))

                    elements=[]
                    for i in tweetformat.split(' '):
                        tmpelement = ''
                        # if i is not an empty string
                        if i:
                            if i.startswith('{') and i.endswith('}'):
                                tmpelement = i.strip('{}')
                                elements.append(tmpelement)

                    # match elements of the tweet format string with available element in the RSS feed
                    fe = FilterEntry(elements, entry, options)
                    entrytosend = fe.finalentry
                    if entrytosend:
                        tweetwithnotag = tweetformat.format(**entrytosend)
                        # remove duplicates from the final tweet
                        dedup = RemoveDuplicates(tweetwithnotag)
                        # only append hashtags if they exist
                        # remove last tags if tweet too long
                        if 'hashtags' in rss:
                            addtag = AddTags(dedup.finaltweet, rss['hashtags'])
                            finaltweet = addtag.finaltweet
                        else:
                            finaltweet = dedup.finaltweet
                        
                    if clioptions.dryrun:
                        if entrytosend:
                            logging.warning('Tweet should have been sent: {tweet}'.format(tweet=finaltweet))
                        else:
                            logging.warning('This rss entry did not meet pattern criteria. Should have not been sent')
                    else:
                        if entrytosend and not clioptions.populate:
                            logging.debug('sending the following tweet:{tweet}'.format(tweet=finaltweet))
                            TweetPost(config, finaltweet)
                        else:
                            logging.debug('populating RSS entry {}'.format(rss['id']))
                        # in both cas we store the id of the sent tweet
                        cache.append(rss['id'])
                        # plugins
                        if plugins and entrytosend:
                            for plugin in plugins:
                                capitalizedplugin = plugin.title()
                                pluginclassname = '{plugin}Plugin'.format(plugin=capitalizedplugin)
                                pluginmodulename = 'feed2tweet.plugins.{pluginmodule}'.format(pluginmodule=pluginclassname.lower())
                                try:
                                    pluginmodule = importlib.import_module(pluginmodulename)
                                    pluginclass = getattr(pluginmodule, pluginclassname) 
                                    pluginclass(plugins[plugin], finaltweet)
                                except ImportError as err:
                                    print(err)
            # do not forget to close cache (shelf object)
            cache.close()



# -*- coding: utf-8 -*-
# Copyright © 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get values of the configuration file
'''Get values of the configuration file'''

# standard library imports
from configparser import SafeConfigParser, NoOptionError, NoSectionError
import os
import os.path
import sys

# 3rd party library imports
import feedparser

class ConfParse(object):
    '''ConfParse class'''
    def __init__(self, clioptions):
        '''Constructor of the ConfParse class'''
        self.clioptions = clioptions
        self.tweetformat = ''
        self.stringsep = ','
        self.confs = []
        self.main()

    def main(self):
        '''Main of the ConfParse class'''
        for pathtoconfig in self.clioptions.configs:
            options = {}
            # read the configuration file
            config = SafeConfigParser()
            if not config.read(os.path.expanduser(pathtoconfig)):
                sys.exit('Could not read config file')

            # get the format of the tweet
            section = 'rss'
            if config.has_section(section):
                # tweet option
                confoption = 'tweet'
                if config.has_option(section, confoption):
                    self.tweetformat = config.get(section, confoption)
                else:
                    sys.exit('You should define a format for your tweet with the keyword "tweet" in the [rss] section')

                # pattern format option
                options['patterns'] = {}
                options['patternscasesensitive'] = {}
                for pattern in ['summary_detail', 'published_parsed', 'guidislink', 'authors', 'links', 'title_detail', 'author', 'author_detail', 'comments', 'published', 'summary', 'tags', 'title', 'link', 'id']:
                    currentoption = '{}_pattern'.format(pattern)
                    if config.has_option(section, currentoption):
                        tmppattern = config.get(section, currentoption)
                        if self.stringsep in tmppattern:
                            options['patterns'][currentoption] = [i for i in tmppattern.split(self.stringsep) if i]
                        else:
                            options['patterns'][currentoption] = [tmppattern]

                    # pattern_case_sensitive format
                    currentoption = '{}_pattern_case_sensitive'.format(pattern)
                    if config.has_option(section, currentoption):
                        try:
                            options['patternscasesensitive'][currentoption] = config.getboolean(section, currentoption)
                        except ValueError as _:
                            options['patternscasesensitive'][currentoption] = True

                # check if options['patterns'] always has a counterpart to True in options['patternscasesensitive']
                for patternoption in options['patterns']:
                    if patternoption not in options['patternscasesensitive']:
                        options['patternscasesensitive']['{pattern}_case_sensitive'.format(pattern=patternoption)] = True

                # rsslist
                feeds = []
                currentoption = 'uri_list'
                if config.has_option(section, currentoption):
                    rssfile = config.get(section, currentoption)
                    rsslist = open(rssfile, 'r').readlines()
                    rsslist =  (i.strip() for i in rsslist if i)
                    for rss in rsslist:
                        feeds.append(feedparser.parse(rss))

                # uri
                if not feeds and not self.clioptions.rss_uri:
                    confoption = 'uri'
                    if config.has_option(section, confoption):
                        options['rss_uri'] = config.get('rss', 'uri')
                    else:
                        sys.exit('{confoption} parameter in the [{section}] section of the configuration file is mandatory. Exiting.'.format(section=section, confoptionn=confoption))
                else:
                    options['rss_uri'] = self.clioptions.rss_uri
                # get the rss feed for rss parameter of [rss] section
                feed = feedparser.parse(options['rss_uri'])

            # cache section
            section = 'cache'
            if not self.clioptions.cachefile:
                confoption = 'cachefile'
                if config.has_section(section):
                    options['cachefile'] = config.get(section, confoption)
                else:
                    sys.exit('You should provide a {confoption} parameter in the [{section}] section'.format(section=section, confoption=confoption))
                if not os.path.isabs(options['cachefile']):
                    sys.exit('You should provide an absolute path to the cache file in the [{section}] section'.format(section=section))
            else:
                options['cachefile'] = self.clioptions.cachefile

            # hashtaglist section
            section = 'hashtaglist'
            if not self.clioptions.hashtaglist:
                confoption = 'several_words_hashtags_list'
                if config.has_section(section):
                    options['hashtaglist'] = config.get(section, confoption)
                else:
                    options['hashtaglist'] = False

            # plugins section
            plugins = {}
            section = 'influxdb'
            if config.has_section(section):
                plugins[section] = {}
                for currentoption in ['host','port','user','pass','database']:
                    if config.has_option(section, currentoption):
                        plugins[section][currentoption] = config.get(section, currentoption)
                if 'host' not in plugins[section]:
                    plugins[section]['host'] = '127.0.0.1'
                if 'port' not in plugins[section]:
                    plugins[section]['port'] = 8086
                if 'measurement' not in plugins[section]:
                    plugins[section]['measurement'] = 'tweets'
                for field in ['user','pass','database']:
                    if field not in plugins[section]:
                        sys.exit('Parsing error for {field} in the [{section}] section: {field} is not defined'.format(field=field, section=section))

            # storing results of the parsing
            if feeds:
                self.confs.append((options, config, self.tweetformat, feeds, plugins))
            else:
                self.confs.append((options, config, self.tweetformat, [feed], plugins))
        
    @property
    def confvalues(self):
        '''Return the values of the different configuration files'''
        return self.confs



# Copyright 2015-2016 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
#!/usr/bin/env python3

# Setup for Feed2tweet
'''Setup for Feed2tweet'''

from setuptools import setup, find_packages

CLASSIFIERS = [
    'Intended Audience :: End Users/Desktop',
    'Environment :: Console',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Operating System :: POSIX :: Linux',
    'Programming Language :: Python :: 3.4'
]


setup(
    name='feed2tweet',
    version='0.9',
    license='GNU GPL v3',
    description='Parse rss feed and tweet new posts to Twitter',
    long_description='Parse rss feed and tweet new posts to Twitter',
    author = 'Carl Chenet',
    author_email = 'chaica@ohmytux.com',
    url = 'https://github.com/chaica/feed2tweet',
    classifiers=CLASSIFIERS,
    download_url='https://github.com/chaica/feed2tweet',
    packages=find_packages(),
    scripts=['scripts/feed2tweet'],
    install_requires=['feedparser', 'persistentlist>=0.4', 'tweepy'],
    extras_require={
        'influxdb':  ["influxdb"]
    }
)
